<?php

/**
 * Provides an interface for creating additional Approval Action types.
 * An approval action represents a point at which some entity may potentially
 * modify a request and then make a decision about the request (by modifying
 * its status).  A Model implementing this interface will be a child of Unl_Model.
 */

interface Creq_Model_ApprovalActionInterface
{
	/**
     * Returns the human friendly name of the given action.
     *
     * @return String
     */
    public function getName();
    
    /**
     * Reutrns an array of strings, corresponding to the possible
     * results of performing this action.  If a customized action can only
     * result a subset of the possible results, return that subset.
     *
     * @return array
     */
    public function getResultStatusStrings();

    /**
     * Returns the XHTML template used to edit the action's specific details
     *
     * @return String
     */
    public function getEditTemplate();
    
    /**
     * Initiates the action, causing it to begin processing the given request.
     * When consideration finishes, the parent chain will be called again.
     *
     * @param Request $request
     * @param ApprovalChain $parentChain
     */
    public function consider($request, $parentChain);

    /**
     * Updates the current action object with the provided raw form data.
     *
     * @param Array $formData
     */
    public function updateFromForm($formData);

    /**
     * Let the action know that a user has made a decsion so that the approval
     * process may continue.
     *
     * @param Request $request
     * @param User $user
     * @param string $decision
     */
    public function userMadeDecision($request, $user, $decision);

    /**
     * Returns the decision the user made on this request.
     * Used when a request is not advanced by a single user, and the pending
     * decision needs to be seen.
     * May return null if no decision has been made, or if the user's action
     * always advances the request.
     *
     * @param Request $request
     * @param User $user
     * @return string
     */
    public function getUserDecision($request, $user);
}

