<?php

/**
 * This class handles the generation and scheduling of email notifications
 * that alert users of requests that are pending their decisions.
 */

class Creq_RequestNotification
{
	/**
	 * Stores the list of notifications once it has been generated to avoid
	 * needlessly reproccessing it.
	 * 
	 * @var array
	 */
	static protected $_notifications;
	
	/**
	 * Does the work of processing which users have pending requests and populates
	 * $_notifications with the list.
	 */
    static protected function _loadNotifications()
    {
    	if (Unl_Util::isArray(self::$_notifications)) {
    		return;
    	}
    	self::$_notifications = array();
    	
        $users = Auth_UserModel::findAll();
        $userRoles = Requests_ApprovalRoleModel::findByUser($users);
        
        $allRoles = Requests_ApprovalRoleModel::findAll();
        $ownedRequests = Requests_RequestModel::findByUser($users);
        $roleRequests = Requests_RequestModel::findByRole($allRoles);
        
        foreach ($users as $user) {
            $myRoles = $userRoles[$user->getId()];
            unset($myRoles[1]);
            $myRequests = new Unl_Model_Collection('Requests_RequestModel');
            foreach ($myRoles as $role) {
                if ($roleRequests[$role->getId()]) {
                    $myRequests->merge($roleRequests[$role->getId()]);
                }
            }
            $myOwnedRequests = $ownedRequests[$user->getId()];
            $myOwnedRequestsToUnset = array();
            foreach ($myOwnedRequests as $id => $myOwnedRequest) {
                if (!$myOwnedRequest->isSubmitterAttentionRequired()) {
                    $myOwnedRequestsToUnset[] = $id;
                }
            }
            foreach ($myOwnedRequestsToUnset as $id) {
            	unset ($myOwnedRequests[$id]);
            }
            $myRequests->merge($myOwnedRequests);
            
            
            $myVotes = Requests_ApproverVoteModel::findUsersVotesForRequests($user, $myRequests);
            $myRequestCount = count($myRequests);
            $myVotedRequests = array();
            
            if (count($myRequests) > 0) {
                $myRequestStacks = Requests_StackModel::findForRequests($myRequests);
            }
            
            foreach ($myVotes as $myVote) {
            	if ($myVote->getApprovalAction() != $myRequestStacks[$myVote->getRequest()]->getAction()) {
            		continue;
            	}
            	
            	// THIS IS A HACK TO EXCLUDE PRE-VOTES FROM PREVIOUSLY TABLED VOTES OF THE SAME ACTION.
            	// IF ANY [VOTING PERIOD/TIME BETWEEN VOTING PERIODS] IS EVER [LONGER/SHORTER] THAN 14 DAYS IT WILL NO LONGER WORK!
            	if ($myVote->getTime()->getTimestamp() < time() - 60*60*24*14) {
            		continue;
            	}
                $myVotedRequests[] = $myVote->getRequest();
            }
            
            if ($myRequestCount <= 0) {
                continue;
            }
            foreach ($myRequests as $key => $myRequest) {
                if (in_array($myRequest->getId(), $myVotedRequests)) {
                    $myRequestCount--;
                }
            }
            
            if ($myRequestCount <= 0) {
                continue;
            }
            
            self::$_notifications[$user->getId()] = array(
                'user'     => $user,
                'requestCount' => $myRequestCount
            );
        }
    }
    
    /**
     * Prints out a simple list of the notifications in plain text.
     * Mainly useful when debugging.
     */
    public static function listNotifications()
    {
    	self::_loadNotifications();
    	
    	foreach (self::$_notifications as $userNotification) {
    		$user = $userNotification['user'];
    		$requestCount = $userNotification['requestCount'];
    		
    		echo $user->getUsername() . ' has ' . $requestCount . ' pending requests.' . "\n";
    	}
    }
    
    public static function getNotifications()
    {
        self::_loadNotifications();
    	
        return self::$_notifications;
    }
    
    /**
     * Sends out an email to each user who has pending requests.
     */
    public static function sendNotifications()
    {
        self::_loadNotifications();
        
	    $messageTextModel = RegistryModel::findByNamespaceAndKey('Email Notification', 'message-text');
	    $messageHtmlModel = RegistryModel::findByNamespaceAndKey('Email Notification', 'message-html');
	    $messageText = $messageTextModel->getData();
	    $messageHtml = $messageHtmlModel->getData();
        
        foreach (self::$_notifications as $userNotification) {
            $user = $userNotification['user'];
            $requestCount = $userNotification['requestCount'];
            
            // If we don't have a user's email address, we can't send them anything.
            if (!$user->getEmail()) {
                continue;
            }
            
            $mail = new Zend_Mail();
            $mail->addTo($user->getEmail(), $user->getFirstName() . ' ' . $user->getLastName());
            $mail->setFrom('automated@courseapproval.unl.edu', 'Course Approval');
            $mail->setSubject('Curriculum Approval: You have ' . $requestCount . ' pending requests.');
            $mail->setBodyText($messageText);
            $mail->setBodyHtml($messageHtml);
            $mail->send();
        }
    }
    
    /**
     * Sends out email notifications, then schedules itself to be called again in two days.
     * This should really only be called by a cron job.
     */
    public static function scheduledNotifications()
    {
    	self::sendNotifications();

        $cronJob = Cron_JobsModel::fetchNew();
        $cronDate = new Zend_Date();
        $cronDate->addDay(2);
        $cronDate->setHour(1);
        $cronDate->setMinute(0);
        $cronDate->setSecond(0);
        $cronJob->setDate($cronDate);
        $cronJob->setClass(__CLASS__);
        $cronJob->setMethod(__FUNCTION__);
        
        $collection = new Unl_Model_Collection('Cron_JobsModel');
        $collection[] = $cronJob;
        Cron_JobsModel::save($collection);
    }
}